import numpy as np
from skcomponents.optics.component import OpticalComponent
from skcomponents.optics.polarizers import VerticalPolarizer, HorizontalPolarizer
from skcomponents.optics.rotators import LinearRetarder, HalfWavePlate


class AOTF(OpticalComponent):

    def __init__(self, on: bool = True):

        self._state = on
        self._aotf = (LinearRetarder(rotation=45, phase=180) +
                      LinearRetarder(rotation=0, phase=180) +
                      HorizontalPolarizer())

    @property
    def on(self):
        return self._state

    @on.setter
    def on(self, value):
        if type(value) is bool:
            self._state = value
        else:
            raise ValueError('only True/False are accepted')

    def matrix(self, wavelength):
        if self.on:
            return self._aotf.matrix(wavelength)
        else:
            return np.eye(4)
