from skcomponents.optics import VerticalPolarizer, HorizontalPolarizer
from sasktran import StokesVector, Geometry, LineOfSight
from skretrieval.core import OpticalGeometry
import numpy as np
import pytest


def test_vertical_polarizer(optical_geometry, model_wavelength, model_geometry, radiance):

    vp = VerticalPolarizer()
    rad = vp.model_radiance(optical_geometry, model_wavelength, model_geometry, radiance)
    assert np.allclose(rad[0][0]._stokes, np.array([1, -1, 0, 0.0]))
    assert np.allclose(rad[0][1]._stokes, np.array([0.0, 0.0, 0, 0.0]))
    assert np.allclose(rad[0][2]._stokes, np.array([0.5, -0.5, 0, 0.0]))
    assert np.allclose(rad[0][3]._stokes, np.array([0.5, -0.5, 0, 0.0]))


def test_polarizer_dataset(optical_geometry, model_wavelength, model_geometry, radiance_dataset):

    vp = VerticalPolarizer()
    rad = vp.model_radiance(optical_geometry, model_wavelength, model_geometry, radiance_dataset)

    print('I:', rad.I.values[0])
    print('Q:', rad.Q.values[0])
    print('U:', rad.U.values[0])
    print('V:', rad.V.values[0])
    assert np.allclose(rad.I.values[0], np.array([1, 0, 0.5, 0.5]))


def test_horizontal_polarizer(optical_geometry, model_wavelength, model_geometry, radiance):

    vp = HorizontalPolarizer()
    rad = vp.model_radiance(optical_geometry, model_wavelength, model_geometry, radiance)
    assert np.allclose(rad[0][1]._stokes, np.array([1, 1, 0, 0.0]))
    assert np.allclose(rad[0][3]._stokes, np.array([0.5, 0.5, 0, 0.0]))