from skcomponents.optics import QuarterWavePlate, IdealLCR, HalfWavePlate
from sasktran import StokesVector, Geometry, LineOfSight
from skretrieval.core import OpticalGeometry
import numpy as np
import pytest


def test_quarterwave_plate(optical_geometry, model_wavelength, model_geometry, radiance):

    qwp = QuarterWavePlate(rotation=0)
    rad = qwp.model_radiance(optical_geometry, model_wavelength, model_geometry, radiance)
    print(qwp.matrix(500))
    assert np.allclose(rad[0][0]._stokes, np.array([1, -1, 0, 0]))
    assert np.allclose(rad[0][1]._stokes, np.array([1, 1, 0, 0]))
    assert np.allclose(rad[0][2]._stokes, np.array([1, 0, 0, -1]))
    assert np.allclose(rad[0][3]._stokes, np.array([1, 0, 0, 1]))


def test_halfwave_plate(optical_geometry, model_wavelength, model_geometry, radiance):

    hwp = HalfWavePlate(rotation=0)
    rad = hwp.model_radiance(optical_geometry, model_wavelength, model_geometry, radiance)
    print(rad[0][0])
    assert np.allclose(rad[0][0]._stokes, np.array([1, -1, 0, 0]))
    assert np.allclose(rad[0][1]._stokes, np.array([1, 1, 0, 0]))
    assert np.allclose(rad[0][2]._stokes, np.array([1, 0, 1, 0]))
    assert np.allclose(rad[0][3]._stokes, np.array([1, 0, -1, 0]))


def test_ideal_lcr(optical_geometry, model_wavelength, model_geometry, radiance):

    lcr = IdealLCR(twist_angle=90)
    rad = lcr.model_radiance(optical_geometry, model_wavelength, model_geometry, radiance)
    print('matrix:', lcr.matrix(500))
    print('radiance:', rad[0][0])

    assert np.allclose(rad[0][0]._stokes, np.array([1, 1, 0, 0]))
    assert np.allclose(rad[0][1]._stokes, np.array([1, -1, 0, 0]))
    assert np.allclose(rad[0][2]._stokes, np.array([1, 0, 1, 0]))
    assert np.allclose(rad[0][3]._stokes, np.array([1, 0, -1, 0]))
